use strict;
use Test::Simple tests => 33;
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::Object::API::Factory qw( ExistsObject LoadRootObject LoadClassByAlias ExistsClassByAlias );

my $ATTRIBUTE_PACKAGE = 'DE_EPAGES::Object::API::Attributes::ReferenceLoop';

sub Main {
    ConnectTestSite();

    # create test classes
    ok( Init(), "initialization" );
    my $Root = LoadRootObject();
    my $ClassC1 = LoadClassByAlias('C1');
    my $ClassC2 = LoadClassByAlias('C2');
    my $ClassC3 = LoadClassByAlias('C3');
    my $ClassC4 = LoadClassByAlias('C4');

    my $Object101 = $ClassC1->insertObject({ Parent => $Root,         Alias => 'C1O1' } );
    my $Object102 = $ClassC1->insertObject({ Parent => $Object101,    Alias => 'C1O2' } );
    my $Object201 = $ClassC2->insertObject({ Parent => $Object102,    Alias => 'C2O1' } );
    my $Object301 = $ClassC3->insertObject({ Parent => $Root,         Alias => 'C3O1' } );
    my $Object401 = $ClassC4->insertObject({ Parent => $Object102,    Alias => 'C4O1' } );
    my $Object402 = $ClassC4->insertObject({ Parent => $Object102,    Alias => 'C4O2' } );

    ok( $Object301->id, "InsertObject" );

    # set attributes
    $Object101->set( { 'A1' => [ $Object102 ] } );
    ok( $Object101->get( 'A1' )->[0]->id == $Object102->id, "get() + set()" );

    # set wrong type
    eval { $Object101->set( { 'A1' => [ $Object201 ] } ) };
    if( ExistsError() ) {
        ok( GetError()->code eq 'WrongValueType', "set wrong type" );
    }
    else {
        ok( 0, "set wrong type - type check failed" );
    }

    ok( $Object101->get('A1')->[0]->id == $Object102->id, "attribute value has not changed" );

    # undefined attributes
    ok( 0 == scalar @{ $Object101->get('A2') }, "undefined get()" );

    # setXXX
    $Object102->set({'A2'=>[ $Object201, $Object301, $Object401, $Object402 ]});
    my $A2 = $Object102->get('A2');
    ok( 4 == scalar @$A2, "get/set 1" );
    ok( $A2->[0]->id == $Object201->id, "get/set 2" );
    ok( $A2->[3]->id == $Object402->id, "get/set 3" );

    $Object102->set({'A3' => [$Object301, $Object401, $Object402]});

    ok( $Object102->get('A3')->[1]->id == $Object401->id, "get()->[1]" );
    ok( $Object102->get('A3')->[2]->id == $Object402->id, "get()->[2]" );

    # empty loop
    $Object102->set({'A2' => []});
    ok( 0 == scalar @{ $Object102->get('A2') }, "empty loop 1" );

    # delete object
    ok( 3 == scalar @{ $Object102->get('A3') }, "before delete object" );
    ok( ExistsObject( $Object401->id ), "object exists" );
    my $ObjectID401 = $Object401->id;
    $Object401->delete;
    ok( !ExistsObject( $ObjectID401 ), "object is deleted" );
    ok( 2 == scalar @{ $Object102->get('A3') }, "after delete object" );

    # delete value
    $Object102->set({'A2'=> [ $Object201 ]});
    ok( 1 == scalar @{ $Object102->get('A2') }, "before delete value" );
    $Object102->set({'A2'=>undef});
    ok( 0 == scalar @{ $Object102->get('A2') }, "after delete value" );
    ok( ExistsObject( $Object201->id ), "referenced object still exists" );

    # delete attribute
    $Object102->set({'A2'=>[ $Object201 ]});
    ok( 1 == scalar @{ $Object102->get('A2') }, "before delete attribute" );
    ok( $Object102->class->existsAttribute('A2'), "before delete attribute 2" );
    $Object102->class->deleteAttribute('A2');
    ok( !$Object102->class->existsAttribute('A2'), "after delete attribute" );
    ok( ExistsObject( $Object201->id ), "referenced object still exists" );

    # delete class
    ok( 2 == scalar @{ $Object102->get('A3') }, "before delete class" );
    my $Object402ID = $Object402->id;
    ok( ExistsObject( $Object402ID ), "instance object exists" );
    $Object402->class->delete;
    ok( 1 == scalar @{ $Object102->get('A3') }, "after delete class" );
    ok( !ExistsObject( $Object402ID ), "instance object is deleted too" );

    # set a new value for A3
    $Object102->set({'A3'=>[ $Object301 ]});

    # delete parent
    ok( 1 == scalar @{ $Object102->get('A3') }, "before delete parent" );
    ok( ExistsObject( $Object301->id ), "referenced object exists" );
    my $ObjectID101 = $Object101->id;
    my $ObjectID102 = $Object102->id;
    $Object101->delete;
    ok( !ExistsObject( $ObjectID101 ), "after delete parent" );
    ok( !ExistsObject( $ObjectID102 ), "child object is deleted too" );
    ok( ExistsObject( $Object301->id ), "referenced object still exists" );

    ok( Cleanup(), "cleanup" );

    DisconnectSite();
}

sub Cleanup {
    foreach my $ClassName ( qw( C3 C2 C1 ) ) {
        next unless ExistsClassByAlias( $ClassName );
        LoadClassByAlias( $ClassName )->delete;
    }
    return 1;
}

sub Init {
    Cleanup();
    my $ClassObject = LoadClassByAlias( 'Object' );
    my $Class1 = $ClassObject->insertSubClass({ Alias => 'C1' });
    $ClassObject->insertSubClass({ Alias => 'C2' });
    my $Class3 = $ClassObject->insertSubClass({ Alias => 'C3' });
    $Class3->insertSubClass({ Alias => 'C4' });
    $Class1->insertAttribute({ Alias => 'A1', Type => $Class1->alias, Package => $ATTRIBUTE_PACKAGE });
    $Class1->insertAttribute({ Alias => 'A2', Type => 'Object',              Package => $ATTRIBUTE_PACKAGE });
    $Class1->insertAttribute({ Alias => 'A3', Type => $Class3->alias, Package => $ATTRIBUTE_PACKAGE });
    return 1;
}

RunScript(Sub=>\&Main, Debug=>1);
