use Test::More;
use strict;
use Time::HiRes qw( gettimeofday tv_interval );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw( ExistsObjectByPath LoadObjectByPath );
use DE_EPAGES::WebInterface::API::FakeRequest;

my $SHOPPATH = "/Shops/DemoShop";

sub LoginToshop {
    my ($Shop, $URL) = @_;

    my $FakeRequest = DE_EPAGES::WebInterface::API::FakeRequest->new;

    my $UrlParams = join('&', (
        'ChangeAction=SaveLoginForm',
        'Login=admin',
        'Password=admin'
    ));

    my $Response = $FakeRequest->get(
        $URL . '?' . $UrlParams,
    );
    cmp_ok( $Response->code, '==', 200, 'Login to shop response 200 OK' );

    # grep URL with sec-Token inside from content (LWP::UserAgent would do this aswell ...)
    # i.e.: <base href="https://qa-vm-lin64-1.intern.epages.de/epages/Store.admin/de_DE/">
    my ($SessionURL) = $Response->content =~ /<base\s*href=["'](.*)["']>/mgx;   #'
    if(!$SessionURL || $SessionURL eq '') {
        # alternatively we have a javascript var for that URL
        # i.e.: epages.vars.BaseUrl = 'http://slange.intern.epages.de/epages/DemoShop.admin/sec7f9ebdacf6/';
        ($SessionURL) = $Response->content =~ /epages.vars.BaseUrl\s*=\s*['"](.*)['"]/mgx;     #"
    }
    ok( $SessionURL =~ /DemoShop\.admin\/sec/g, 'Search for session URL in content');

    return $SessionURL;
}

sub DoSmartSearch {
    my ($Shop, $SessionURL, $SearchString) = @_;

    my $FakeRequest = DE_EPAGES::WebInterface::API::FakeRequest->new;

    my $UrlParams = join('&', (
        'ViewAction=JSONSmartSearchProductAlias',
        'ObjectID='.$Shop->id,
        'SearchString='.$SearchString
    ));

    my $Response = $FakeRequest->get(
        $SessionURL . '?' . $UrlParams,
        [
            'content-type' => 'application/x-www-form-urlencoded; charset=UTF-8',
            'Accept' => 'text/x-json',
        ]
    );

    # ok( $Response->is_success, '200 OK' );
    die("smart search return code OK!") unless $Response->is_success;
    # ok( $Response->content =~ /"NumResults"\s:\s/ && $Response->content =~ /"Results"\s:\s/, 'Return value appears JSON' );
    die("smart search returned content not JSON! (is: ".$Response->content.")") unless $Response->content =~ /"NumResults"\s:\s/ && $Response->content =~ /"Results"\s:\s/;
}

sub TestProductSmartSearch {
    my ($Shop, $SessionURL) = @_;
    foreach my $SearchString ( qw(be_)) {
        my $t0= [gettimeofday];
        DoSmartSearch($Shop, $SessionURL, $SearchString);
        my $t1= [gettimeofday];
        my $elapsed = tv_interval ($t0, $t1);
        cmp_ok( $elapsed, '<', 0.2, sprintf("One product alias smart search takes less than 0.2 seconds. This run took: %0.2f seconds.", $elapsed) );
    }
}

sub MassTestProductSmartSearch {
    my ($Shop, $SessionURL) = @_;

    my @SearchQueryPool = qw(be be_ be_4040 be_40401 be_404010100 be_40402 cg cg_ js js_32 md_49417110 md_494171101);
    my $t0= [gettimeofday];
    for (0..200) {
        DoSmartSearch($Shop, $SessionURL, $SearchQueryPool[rand @SearchQueryPool]);
    }
    my $t1= [gettimeofday];
    my $elapsed = tv_interval ($t0, $t1);
    cmp_ok( $elapsed, '<', 15, sprintf("200 random product alias smart searches take less than 15 seconds. This run took: %0.2f seconds.", $elapsed) );
}

sub Main {

    if( ExistsObjectByPath($SHOPPATH) ) {
        my $Shop = LoadObjectByPath($SHOPPATH);
        ok( $Shop->id, "load demoshop " . $Shop->id );
        my $URL = $Shop->get('WebServerScriptAdminUrl');
        my $SessionURL = LoginToshop($Shop, $URL);

        # user clicks on product list and clicks on a product
        LoadObjectByPath($SHOPPATH . '/Products');
        LoadObjectByPath($SHOPPATH . '/Products/be_40401');

        TestProductSmartSearch($Shop, $SessionURL);
        MassTestProductSmartSearch($Shop, $SessionURL);
    } else {
        SKIP: {
            skip "no DemoShop installed", 8;
        };
    }
    return;
}


RunScript( Message => 'long', Sub=> sub {
    RunOnStore( Store => GetTestSite(), Sub => \&Main );
});

done_testing();