use Test::Simple tests => 39;
use strict;
use DE_EPAGES::Core::API::Error qw (ExistsError GetError);
use DE_EPAGES::Core::API::Script qw (RunScript);
use DE_EPAGES::Database::API::Connection qw( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Language qw ( GetPKeyLanguageByCode );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject LoadObjectByPath ExistsObjectByPath );

my $ROOTPATH = '/Test';

sub Main {
    Init();

    my $System = LoadRootObject();
    my $Shop = LoadObjectByPath($ROOTPATH.'/TestShop1');
    ok( $Shop->id, "create shop " . $Shop->id );
    ok( $Shop->instanceOf('Shop'), "Shop is instance of class Shop" );

    ok( !$Shop->get('ClosedByMerchant'), "Shop.getClosedByMerchant" );
    ok( !$Shop->get('ClosedByProvider'), "Shop.getClosedByProvider" );

    $Shop->set({'ClosedByMerchant'=>1});
    ok( $Shop->get('ClosedByMerchant'), "Shop.setClosedByMerchant" );

    $Shop->set({'ClosedByProvider'=>1});
    ok( $Shop->get('ClosedByProvider'), "Shop.setClosedByProvider" );

    ok( 1 == $Shop->featureMaxValue('Languages'), "new shops can have 1 langauge" );
    ok( $Shop->get('LocaleID') eq $System->get('LocaleID'), "new shops use the system locale" );

    ok( 1 == $Shop->featureCurrentValue('Languages'), "new shops have 1 language" );

    # featureIsDirty
    ok( !$Shop->featureIsDirty('Languages'), "Feature Languages is not dirty" );
    ok( 0 == scalar $Shop->featureIsDirty('Languages', 1), "featureIsDirty(feature, value) returns nothing" );
    ok( $Shop->featureIsDirty('Languages'), "featureIsDirty after update" );
    $Shop->featureIsDirty('Languages', 0);
    ok( !$Shop->featureIsDirty('Languages'), "featureIsDirty after reset" );

    # addLanguage with Languages=1
    my $LanguageID = GetPKeyLanguageByCode( 'ger' );
    $Shop->addLanguage($LanguageID);
    ok ( $Shop->existsLanguage($LanguageID) , "existsLanguage" );
    ok ( 1 == scalar @{$Shop->languages}, "Shop.languages" );
    ok ( $LanguageID == $Shop->languages->[0], "Shop.languages[0]" );
    ok ( 'de_DE' eq $Shop->get('LocaleID'), "Shop.LocaleID" );

    # switch language with Languages=1
    $LanguageID = GetPKeyLanguageByCode( 'en' );
    $Shop->addLanguage($LanguageID);
    ok ( $Shop->existsLanguage($LanguageID) , "existsLanguage 2" );
    ok ( 1 == scalar @{$Shop->languages}, "Shop.languages 2" );
    ok ( $LanguageID == $Shop->languages->[0], "Shop.languages[0] 2" );
    ok ( 'en_GB' eq $Shop->get('LocaleID'), "Shop.LocaleID 2" );

    my %BOLocales = map { $_->get('LocaleID') => 1 } @{$Shop->get('BackofficeLocales')};
    ok( !exists $BOLocales{'de_AT'}, "de_AT not in BackofficeLocales" );

    # add locales
    eval { $Shop->addLocale('en_AU'); };
    ok( $Shop->existsLocale('en_AU'), "addLocale en_AU" );
    ok( 2 == scalar @{$Shop->get('SiteLocales')}, "Shop.SiteLocales" );
    eval { $Shop->addLocale('de_DE'); };
    ok( ExistsError() && GetError->code eq 'LocaleDisabled', "add locale from disabled language" );
    eval { $Shop->addLocale('ru_RU'); };
    ok( ExistsError() && GetError->code eq 'LocaleDisabled', "add locale that is not available" );
    eval { $Shop->addLocale('en_UK'); };
    ok( ExistsError() && GetError->code eq 'LocaleDisabled', "add non-existing locale" );

    $Shop->updateSiteLocale('en_AU', 9);
    ok($Shop->localeByLanguage($LanguageID) eq 'en_AU', "first locale is en_AU");
    $Shop->updateSiteLocale('en_GB', 5);
    ok($Shop->localeByLanguage($LanguageID) eq 'en_GB', "now first locale is en_GB");

    $Shop->removeLocale('en_AU');
    ok (!$Shop->existsLocale('en_AU'), "removeLocale en_AU" );

    # add more languages
    $LanguageID = GetPKeyLanguageByCode( 'de' );
    $Shop->featureMaxValue('Languages', 2);
    $Shop->addLanguage($LanguageID);
    ok( $Shop->existsLanguage($LanguageID) , "existsLanguage(de)" );
    ok( 2 == scalar @{$Shop->languages}, "Shop.languages = 2" );
    ok( 'en_GB' eq $Shop->get('LocaleID'), "Shop.LocaleID does not change" );
    %BOLocales = map { $_->get('LocaleID') => 1 } @{$Shop->get('BackofficeLocales')};
    ok( exists $BOLocales{'de_AT'}, "de_AT exists in BackofficeLocales" );

    $Shop->addLocale('de_DE');
    ok( $Shop->existsLocale('de_DE'), "addLocale de_DE" );
    ok( 2 == scalar @{$Shop->get('SiteLocales')}, "Shop.SiteLocales" );

    # add more langauges than allowed
    eval { $Shop->addLanguage(GetPKeyLanguageByCode('fr')); };
    ok( ExistsError() && GetError->code eq 'FeatureLimitExceeded', "add too many languages" );

    # removeLanguage
    $Shop->removeLanguage($LanguageID);
    ok( ! $Shop->existsLanguage($LanguageID), "removeLanguage(de)" );
    ok( ! $Shop->existsLocale('de_DE'), "locales are also removed" );

    # restore defaults
    $Shop->set({
        'ClosedByMerchant' => 0,
        'ClosedByProvider' => 0,
    });
    $Shop->featureMaxValue('Languages', 1);

    Cleanup();
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if LoadRootObject()->existsChild('Test') && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    my $Folder = LoadRootObject()->insertChild( 'Object', { Alias => 'Test' });
    $Folder->insertChild( 'Shop', { Alias => 'TestShop1' } );
    return 1;
}

RunScript( Message => 'long', Sub=> sub {
    RunOnStore( Store => GetTestSite(), Sub => \&Main );
});
